# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.damper import Damper

from .schema_helper import SchemaHelper
from .models_impl import *


class ActiveDamper(Damper, ABC):
    r"""
    An active damper, applying a force which is specified by the controller.
    
    Not supported yet.
    
    Attributes
    ----------
    ForceLag : float, Not supported yet
        The force lag for the force signal's transfer function.
    
    AccelerationLag : float, Not supported yet
        The time lag for the acceleration signal's transfer function.
    
    ForceTransferFunction : float, Not supported yet
        Transfer function for active damper
    
    PerturbationForLinearisationCalculation : float, Not supported yet
        The perturbation step to use during linearisation calculation.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    ForceLag: float = Field(alias="ForceLag", default=None) # Not supported yet
    AccelerationLag: float = Field(alias="AccelerationLag", default=None) # Not supported yet
    ForceTransferFunction: float = Field(alias="ForceTransferFunction", default=None) # Not supported yet
    PerturbationForLinearisationCalculation: float = Field(alias="PerturbationForLinearisationCalculation", default=None) # Not supported yet

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(Damper._type_info)


ActiveDamper.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class WindVerticalShearVariation_VerticalShearVariationTypeEnum(str, Enum):
    PRESET_TRANSIENT = "PresetTransient"
    TIME_HISTORY = "TimeHistory"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *


class WindVerticalShearVariation(BladedModel, ABC):
    r"""
    A defined variation in the vertical wind shear.
    
    Attributes
    ----------
    VerticalShearVariationType : WindVerticalShearVariation_VerticalShearVariationTypeEnum
        Defines the specific type of model in use.
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - WindVerticalShearVariationInsert
        - PresetWindVerticalShearTransient
        - WindVerticalShearTimeHistory
    
    """
    VerticalShearVariationType: WindVerticalShearVariation_VerticalShearVariationTypeEnum = Field(alias="VerticalShearVariationType", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


WindVerticalShearVariation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.exponential_shear_model import ExponentialShearModel
from dnv_bladed_models.logarithmic_shear_model import LogarithmicShearModel
from dnv_bladed_models.look_up_shear_model import LookUpShearModel
from dnv_bladed_models.steady_calculation import SteadyCalculation
from dnv_bladed_models.steady_calculation_with_component_outputs import SteadyCalculationWithComponentOutputs
from dnv_bladed_models.wind_shear import WindShear
from dnv_bladed_models.wind_shear_insert import WindShearInsert
class SteadyParkedLoadsCalculation_SweepParameterEnum(str, Enum):
    AZIMUTH_ANGLE = "AZIMUTH_ANGLE"
    YAW_ANGLE = "YAW_ANGLE"
    FLOW_INCLINATION = "FLOW_INCLINATION"
    PITCH_ANGLE = "PITCH_ANGLE"

from .schema_helper import SchemaHelper
from .models_impl import *

TWindShearOptions = TypeVar('TWindShearOptions', ExponentialShearModel, WindShearInsert, LogarithmicShearModel, LookUpShearModel, WindShear, )

class SteadyParkedLoadsCalculation(SteadyCalculation):
    r"""
    Defines a calculation which produces loads on the parked turbine in a steady wind.  Most realities such as tower shadow and wind shear are included, making the calculation almost equivalent to a time domain simulation in a steady wind.
    
    Not supported yet.
    
    Attributes
    ----------
    SteadyCalculationType : Literal['SteadyParkedLoads'], default='SteadyParkedLoads', Not supported yet
        Defines the specific type of SteadyCalculation model in use.  For a `SteadyParkedLoads` object, this must always be set to a value of `SteadyParkedLoads`.
    
    WindSpeed : float, Not supported yet
        The wind speed at the hub height to be used for the calculation.
    
    AzimuthAngle : float, default=0, Not supported yet
        The fixed azimuth angle of the rotor (zero azimuth indicates blade 1 pointing upwards).
    
    YawAngle : float, default=0, Not supported yet
        The yaw angle to be used for the calculation.
    
    FlowInclination : float, default=0, Not supported yet
        The flow inclination to be used for the calculation.
    
    ReferenceHeight : float, Not supported yet
        The reference height used for wind shear.  If this is omitted, the hub height will be used, and if there is more than one the *highest* hub height.
    
    WindShear : Union[ExponentialShearModel, WindShearInsert, LogarithmicShearModel, LookUpShearModel]
    
    PitchAngle : float, Not supported yet
        The pitch angle of all of the blades to be used for the calculation.  If not provided, the PitchAngleWhilstParked from TurbineOperationalParameters be used.
    
    SweepParameter : SteadyParkedLoadsCalculation_SweepParameterEnum, Not supported yet
        The parameter to perform the sweep over.
    
    SweepEnd : float, Not supported yet
        The value for the end of the sweep.  The start value will be whatever it is in the parameters for the calculation.
    
    SweepInterval : float, Not supported yet
        The step size to take from the lowest to the highest value.
    
    Outputs : SteadyCalculationWithComponentOutputs
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    SteadyCalculationType: Literal['SteadyParkedLoads'] = Field(alias="SteadyCalculationType", default='SteadyParkedLoads', allow_mutation=False, const=True) # Not supported yet # type: ignore
    WindSpeed: float = Field(alias="WindSpeed", default=None) # Not supported yet
    AzimuthAngle: float = Field(alias="AzimuthAngle", default=None) # Not supported yet
    YawAngle: float = Field(alias="YawAngle", default=None) # Not supported yet
    FlowInclination: float = Field(alias="FlowInclination", default=None) # Not supported yet
    ReferenceHeight: float = Field(alias="ReferenceHeight", default=None) # Not supported yet
    WindShear: Union[ExponentialShearModel, WindShearInsert, LogarithmicShearModel, LookUpShearModel] = Field(alias="WindShear", default=None, discriminator='WindShearType')
    PitchAngle: float = Field(alias="PitchAngle", default=None) # Not supported yet
    SweepParameter: SteadyParkedLoadsCalculation_SweepParameterEnum = Field(alias="SweepParameter", default=None) # Not supported yet
    SweepEnd: float = Field(alias="SweepEnd", default=None) # Not supported yet
    SweepInterval: float = Field(alias="SweepInterval", default=None) # Not supported yet
    Outputs: SteadyCalculationWithComponentOutputs = Field(alias="Outputs", default=None)

    _relative_schema_path = 'SteadyCalculation/SteadyParkedLoadsCalculation.json'
    _type_info = TypeInfo(
        set([('WindShear', 'WindShearType'),]),
        set([]),
        set([]),
        'SteadyCalculationType').merge(SteadyCalculation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def WindShear_as_ExponentialShearModel(self) -> ExponentialShearModel:
        """
        Retrieves the value of WindShear guaranteeing it is a ExponentialShearModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ExponentialShearModel
            A model object, guaranteed to be a ExponentialShearModel.

        Raises
        ------
        TypeError
            If the value is not a ExponentialShearModel.
        """
        return self.WindShear_as(ExponentialShearModel)


    @property
    def WindShear_as_LogarithmicShearModel(self) -> LogarithmicShearModel:
        """
        Retrieves the value of WindShear guaranteeing it is a LogarithmicShearModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LogarithmicShearModel
            A model object, guaranteed to be a LogarithmicShearModel.

        Raises
        ------
        TypeError
            If the value is not a LogarithmicShearModel.
        """
        return self.WindShear_as(LogarithmicShearModel)


    @property
    def WindShear_as_LookUpShearModel(self) -> LookUpShearModel:
        """
        Retrieves the value of WindShear guaranteeing it is a LookUpShearModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LookUpShearModel
            A model object, guaranteed to be a LookUpShearModel.

        Raises
        ------
        TypeError
            If the value is not a LookUpShearModel.
        """
        return self.WindShear_as(LookUpShearModel)


    @property
    def WindShear_as_inline(self) -> Union[ExponentialShearModel, LogarithmicShearModel, LookUpShearModel]:
        """
        Retrieves the value of WindShear as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[ExponentialShearModel, LogarithmicShearModel, LookUpShearModel]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of WindShear; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.WindShear, WindShearInsert) or self.WindShear.is_insert:
            raise TypeError(f"Expected WindShear value to be an in-line object, but it is currently in the '$insert' state.")
        return self.WindShear


    def WindShear_as(self, cls: Type[TWindShearOptions])-> TWindShearOptions:
        """
        Retrieves the value of WindShear, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of WindShear, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[ExponentialShearModel, WindShearInsert, LogarithmicShearModel, LookUpShearModel]]
            One of the valid concrete types of WindShear, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWindShearOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of WindShear:
        >>> val_obj = model_obj.WindShear_as(models.ExponentialShearModel)
        >>> val_obj = model_obj.WindShear_as(models.LogarithmicShearModel)
        >>> val_obj = model_obj.WindShear_as(models.LookUpShearModel)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.WindShear_as(models.WindShearInsert)
        """
        if not isinstance(self.WindShear, cls):
            raise TypeError(f"Expected WindShear of type '{cls.__name__}' but was type '{type(self.WindShear).__name__}'")
        return self.WindShear


    def _entity(self) -> bool:
        return True


SteadyParkedLoadsCalculation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.dynamic_wake import DynamicWake
from dnv_bladed_models.dynamic_wake_insert import DynamicWakeInsert
from dnv_bladed_models.equilibrium_wake_model import EquilibriumWakeModel
from dnv_bladed_models.free_flow_model import FreeFlowModel
from dnv_bladed_models.frozen_wake_model import FrozenWakeModel
from dnv_bladed_models.glauert_skew_wake_model import GlauertSkewWakeModel
from dnv_bladed_models.oye_dynamic_wake import OyeDynamicWake
from dnv_bladed_models.pitt_and_peters_model import PittAndPetersModel
class MomentumTheoryCorrections_GlauertCorrectionMethodForHighInductionEnum(str, Enum):
    BLADED = "BLADED"
    NONE = "NONE"

from .schema_helper import SchemaHelper
from .models_impl import *

TDynamicWakeOptions = TypeVar('TDynamicWakeOptions', EquilibriumWakeModel, FreeFlowModel, FrozenWakeModel, DynamicWakeInsert, OyeDynamicWake, PittAndPetersModel, DynamicWake, )

class MomentumTheoryCorrections(BladedModel):
    r"""
    The Blade Element Momentum (BEM) theory model.
    
    Attributes
    ----------
    GlauertCorrectionMethodForHighInduction : MomentumTheoryCorrections_GlauertCorrectionMethodForHighInductionEnum, default='BLADED'
        The Glauert correction method for when the rotor has high induction.
    
    GlauertSkewedWakeCorrectionModel : GlauertSkewWakeModel
    
    InductionFactorsTolerance : float, default=0.00010
        In the steady state (e.g. finding initial conditions) and/or when using equilibrium wake in the dynamic state, the axial and tangential factors are found by iteration. The precision to which these induction factors are found is determined by this tolerance.
    
    NoInflowBelowTipSpeedRatio : float, default=1
        The tip speed ratio below which the inflow calculations will be switched off. Default = 1
    
    FullInflowAboveTipSpeedRatio : float, default=2
        The tip speed ratio above which the inflow calculations will be switched on. Default = 2
    
    DynamicWake : Union[EquilibriumWakeModel, FreeFlowModel, FrozenWakeModel, DynamicWakeInsert, OyeDynamicWake, PittAndPetersModel]
    
    IncludeStructuralVelocityInInductionCalculation : bool, default=True
        If true, the axial structural velocity will be included in the induction calculations. Enabled by default.
    
    Notes
    -----
    
    """
    GlauertCorrectionMethodForHighInduction: MomentumTheoryCorrections_GlauertCorrectionMethodForHighInductionEnum = Field(alias="GlauertCorrectionMethodForHighInduction", default=None)
    GlauertSkewedWakeCorrectionModel: GlauertSkewWakeModel = Field(alias="GlauertSkewedWakeCorrectionModel", default=None)
    InductionFactorsTolerance: float = Field(alias="InductionFactorsTolerance", default=None)
    NoInflowBelowTipSpeedRatio: float = Field(alias="NoInflowBelowTipSpeedRatio", default=None)
    FullInflowAboveTipSpeedRatio: float = Field(alias="FullInflowAboveTipSpeedRatio", default=None)
    DynamicWake: Union[EquilibriumWakeModel, FreeFlowModel, FrozenWakeModel, DynamicWakeInsert, OyeDynamicWake, PittAndPetersModel] = Field(alias="DynamicWake", default=None, discriminator='DynamicWakeType')
    IncludeStructuralVelocityInInductionCalculation: bool = Field(alias="IncludeStructuralVelocityInInductionCalculation", default=None)

    _relative_schema_path = 'Settings/AerodynamicSettings/AerodynamicModel/MomentumTheoryCorrections/MomentumTheoryCorrections.json'
    _type_info = TypeInfo(
        set([('DynamicWake', 'DynamicWakeType'),]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def DynamicWake_as_EquilibriumWakeModel(self) -> EquilibriumWakeModel:
        """
        Retrieves the value of DynamicWake guaranteeing it is a EquilibriumWakeModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        EquilibriumWakeModel
            A model object, guaranteed to be a EquilibriumWakeModel.

        Raises
        ------
        TypeError
            If the value is not a EquilibriumWakeModel.
        """
        return self.DynamicWake_as(EquilibriumWakeModel)


    @property
    def DynamicWake_as_FreeFlowModel(self) -> FreeFlowModel:
        """
        Retrieves the value of DynamicWake guaranteeing it is a FreeFlowModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        FreeFlowModel
            A model object, guaranteed to be a FreeFlowModel.

        Raises
        ------
        TypeError
            If the value is not a FreeFlowModel.
        """
        return self.DynamicWake_as(FreeFlowModel)


    @property
    def DynamicWake_as_FrozenWakeModel(self) -> FrozenWakeModel:
        """
        Retrieves the value of DynamicWake guaranteeing it is a FrozenWakeModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        FrozenWakeModel
            A model object, guaranteed to be a FrozenWakeModel.

        Raises
        ------
        TypeError
            If the value is not a FrozenWakeModel.
        """
        return self.DynamicWake_as(FrozenWakeModel)


    @property
    def DynamicWake_as_OyeDynamicWake(self) -> OyeDynamicWake:
        """
        Retrieves the value of DynamicWake guaranteeing it is a OyeDynamicWake; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        OyeDynamicWake
            A model object, guaranteed to be a OyeDynamicWake.

        Raises
        ------
        TypeError
            If the value is not a OyeDynamicWake.
        """
        return self.DynamicWake_as(OyeDynamicWake)


    @property
    def DynamicWake_as_PittAndPetersModel(self) -> PittAndPetersModel:
        """
        Retrieves the value of DynamicWake guaranteeing it is a PittAndPetersModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PittAndPetersModel
            A model object, guaranteed to be a PittAndPetersModel.

        Raises
        ------
        TypeError
            If the value is not a PittAndPetersModel.
        """
        return self.DynamicWake_as(PittAndPetersModel)


    @property
    def DynamicWake_as_inline(self) -> Union[EquilibriumWakeModel, FreeFlowModel, FrozenWakeModel, OyeDynamicWake, PittAndPetersModel]:
        """
        Retrieves the value of DynamicWake as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[EquilibriumWakeModel, FreeFlowModel, FrozenWakeModel, OyeDynamicWake, PittAndPetersModel]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of DynamicWake; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.DynamicWake, DynamicWakeInsert) or self.DynamicWake.is_insert:
            raise TypeError(f"Expected DynamicWake value to be an in-line object, but it is currently in the '$insert' state.")
        return self.DynamicWake


    def DynamicWake_as(self, cls: Type[TDynamicWakeOptions])-> TDynamicWakeOptions:
        """
        Retrieves the value of DynamicWake, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of DynamicWake, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[EquilibriumWakeModel, FreeFlowModel, FrozenWakeModel, DynamicWakeInsert, OyeDynamicWake, PittAndPetersModel]]
            One of the valid concrete types of DynamicWake, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TDynamicWakeOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of DynamicWake:
        >>> val_obj = model_obj.DynamicWake_as(models.EquilibriumWakeModel)
        >>> val_obj = model_obj.DynamicWake_as(models.FreeFlowModel)
        >>> val_obj = model_obj.DynamicWake_as(models.FrozenWakeModel)
        >>> val_obj = model_obj.DynamicWake_as(models.OyeDynamicWake)
        >>> val_obj = model_obj.DynamicWake_as(models.PittAndPetersModel)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.DynamicWake_as(models.DynamicWakeInsert)
        """
        if not isinstance(self.DynamicWake, cls):
            raise TypeError(f"Expected DynamicWake of type '{cls.__name__}' but was type '{type(self.DynamicWake).__name__}'")
        return self.DynamicWake


    def _entity(self) -> bool:
        return True


MomentumTheoryCorrections.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.additional_constrained_wave import AdditionalConstrainedWave
from dnv_bladed_models.automatic_mac_camy_fuchs import AutomaticMacCamyFuchs
from dnv_bladed_models.automatic_simple_cutoff_frequency import AutomaticSimpleCutoffFrequency
from dnv_bladed_models.mac_camy_fuchs import MacCamyFuchs
from dnv_bladed_models.simple_cutoff_frequency import SimpleCutoffFrequency
from dnv_bladed_models.wave_diffraction_approximation import WaveDiffractionApproximation
from dnv_bladed_models.wave_diffraction_approximation_insert import WaveDiffractionApproximationInsert
from dnv_bladed_models.waves import Waves

from .schema_helper import SchemaHelper
from .models_impl import *

TWaveDiffractionApproximationOptions = TypeVar('TWaveDiffractionApproximationOptions', AutomaticMacCamyFuchs, AutomaticSimpleCutoffFrequency, WaveDiffractionApproximationInsert, MacCamyFuchs, SimpleCutoffFrequency, WaveDiffractionApproximation, )

class IrregularWaves(Waves, ABC):
    r"""
    The definition of irregular waves.
    
    Not supported yet.
    
    Attributes
    ----------
    DirectionOfApproachClockwiseFromNorth : float, Not supported yet
        The bearing from which waves arrive at the turbine.
    
    RandomNumberSeed : int, Not supported yet
        A arbitrary integer used to generate a realisation of the irregular waves.  This ensures that the 'randomness' is consistent from simulation to simulation.
    
    WaveDiffractionApproximation : Union[AutomaticMacCamyFuchs, AutomaticSimpleCutoffFrequency, WaveDiffractionApproximationInsert, MacCamyFuchs, SimpleCutoffFrequency], Not supported yet
    
    AdditionalConstrainedWave : AdditionalConstrainedWave, Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    DirectionOfApproachClockwiseFromNorth: float = Field(alias="DirectionOfApproachClockwiseFromNorth", default=None) # Not supported yet
    RandomNumberSeed: int = Field(alias="RandomNumberSeed", default=None) # Not supported yet
    WaveDiffractionApproximation: Union[AutomaticMacCamyFuchs, AutomaticSimpleCutoffFrequency, WaveDiffractionApproximationInsert, MacCamyFuchs, SimpleCutoffFrequency] = Field(alias="WaveDiffractionApproximation", default=None, discriminator='WaveDiffractionApproximationType') # Not supported yet
    AdditionalConstrainedWave: AdditionalConstrainedWave = Field(alias="AdditionalConstrainedWave", default=None) # Not supported yet

    _type_info = TypeInfo(
        set([('WaveDiffractionApproximation', 'WaveDiffractionApproximationType'),]),
        set([]),
        set([]),
        None).merge(Waves._type_info)


    @property
    def WaveDiffractionApproximation_as_AutomaticMacCamyFuchs(self) -> AutomaticMacCamyFuchs:
        """
        Retrieves the value of WaveDiffractionApproximation guaranteeing it is a AutomaticMacCamyFuchs; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        AutomaticMacCamyFuchs
            A model object, guaranteed to be a AutomaticMacCamyFuchs.

        Raises
        ------
        TypeError
            If the value is not a AutomaticMacCamyFuchs.
        """
        return self.WaveDiffractionApproximation_as(AutomaticMacCamyFuchs)


    @property
    def WaveDiffractionApproximation_as_AutomaticSimpleCutoffFrequency(self) -> AutomaticSimpleCutoffFrequency:
        """
        Retrieves the value of WaveDiffractionApproximation guaranteeing it is a AutomaticSimpleCutoffFrequency; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        AutomaticSimpleCutoffFrequency
            A model object, guaranteed to be a AutomaticSimpleCutoffFrequency.

        Raises
        ------
        TypeError
            If the value is not a AutomaticSimpleCutoffFrequency.
        """
        return self.WaveDiffractionApproximation_as(AutomaticSimpleCutoffFrequency)


    @property
    def WaveDiffractionApproximation_as_MacCamyFuchs(self) -> MacCamyFuchs:
        """
        Retrieves the value of WaveDiffractionApproximation guaranteeing it is a MacCamyFuchs; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        MacCamyFuchs
            A model object, guaranteed to be a MacCamyFuchs.

        Raises
        ------
        TypeError
            If the value is not a MacCamyFuchs.
        """
        return self.WaveDiffractionApproximation_as(MacCamyFuchs)


    @property
    def WaveDiffractionApproximation_as_SimpleCutoffFrequency(self) -> SimpleCutoffFrequency:
        """
        Retrieves the value of WaveDiffractionApproximation guaranteeing it is a SimpleCutoffFrequency; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        SimpleCutoffFrequency
            A model object, guaranteed to be a SimpleCutoffFrequency.

        Raises
        ------
        TypeError
            If the value is not a SimpleCutoffFrequency.
        """
        return self.WaveDiffractionApproximation_as(SimpleCutoffFrequency)


    @property
    def WaveDiffractionApproximation_as_inline(self) -> Union[AutomaticMacCamyFuchs, AutomaticSimpleCutoffFrequency, MacCamyFuchs, SimpleCutoffFrequency]:
        """
        Retrieves the value of WaveDiffractionApproximation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[AutomaticMacCamyFuchs, AutomaticSimpleCutoffFrequency, MacCamyFuchs, SimpleCutoffFrequency]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of WaveDiffractionApproximation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.WaveDiffractionApproximation, WaveDiffractionApproximationInsert) or self.WaveDiffractionApproximation.is_insert:
            raise TypeError(f"Expected WaveDiffractionApproximation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.WaveDiffractionApproximation


    def WaveDiffractionApproximation_as(self, cls: Type[TWaveDiffractionApproximationOptions])-> TWaveDiffractionApproximationOptions:
        """
        Retrieves the value of WaveDiffractionApproximation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of WaveDiffractionApproximation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[AutomaticMacCamyFuchs, AutomaticSimpleCutoffFrequency, WaveDiffractionApproximationInsert, MacCamyFuchs, SimpleCutoffFrequency]]
            One of the valid concrete types of WaveDiffractionApproximation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWaveDiffractionApproximationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of WaveDiffractionApproximation:
        >>> val_obj = model_obj.WaveDiffractionApproximation_as(models.AutomaticMacCamyFuchs)
        >>> val_obj = model_obj.WaveDiffractionApproximation_as(models.AutomaticSimpleCutoffFrequency)
        >>> val_obj = model_obj.WaveDiffractionApproximation_as(models.MacCamyFuchs)
        >>> val_obj = model_obj.WaveDiffractionApproximation_as(models.SimpleCutoffFrequency)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.WaveDiffractionApproximation_as(models.WaveDiffractionApproximationInsert)
        """
        if not isinstance(self.WaveDiffractionApproximation, cls):
            raise TypeError(f"Expected WaveDiffractionApproximation of type '{cls.__name__}' but was type '{type(self.WaveDiffractionApproximation).__name__}'")
        return self.WaveDiffractionApproximation


IrregularWaves.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.angle_range import AngleRange
from dnv_bladed_models.dimensionless_range import DimensionlessRange
from dnv_bladed_models.steady_calculation import SteadyCalculation
from dnv_bladed_models.steady_calculation_outputs import SteadyCalculationOutputs

from .schema_helper import SchemaHelper
from .models_impl import *


class PerformanceCoefficientsCalculation(SteadyCalculation):
    r"""
    Defines a calculation which produces power, torque and thrust coefficients are calculated in a uniform steady wind field.  The rotor is modelled with flexibility as well as prebend and sweep.  However, the rotor is analysed in isolation without any influence from the rest of the turbine structure.
    
    Attributes
    ----------
    SteadyCalculationType : Literal['PerformanceCoefficients'], default='PerformanceCoefficients'
        Defines the specific type of SteadyCalculation model in use.  For a `PerformanceCoefficients` object, this must always be set to a value of `PerformanceCoefficients`.
    
    TipSpeedRatioRange : DimensionlessRange
    
    PitchRange : AngleRange
    
    RotorSpeed : float
        The rotor speed to be considered for the calculation.
    
    Outputs : SteadyCalculationOutputs
    
    Notes
    -----
    
    """
    SteadyCalculationType: Literal['PerformanceCoefficients'] = Field(alias="SteadyCalculationType", default='PerformanceCoefficients', allow_mutation=False, const=True) # type: ignore
    TipSpeedRatioRange: DimensionlessRange = Field(alias="TipSpeedRatioRange", default=None)
    PitchRange: AngleRange = Field(alias="PitchRange", default=None)
    RotorSpeed: float = Field(alias="RotorSpeed", default=None)
    Outputs: SteadyCalculationOutputs = Field(alias="Outputs", default=None)

    _relative_schema_path = 'SteadyCalculation/PerformanceCoefficientsCalculation.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'SteadyCalculationType').merge(SteadyCalculation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


PerformanceCoefficientsCalculation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.wind_direction_shear_variation import WindDirectionShearVariation
class PresetWindDirectionShearTransient_VariationShapeEnum(str, Enum):
    FULL = "FULL"
    HALF = "HALF"
    IEC_2 = "IEC-2"

from .schema_helper import SchemaHelper
from .models_impl import *


class PresetWindDirectionShearTransient(WindDirectionShearVariation):
    r"""
    A preset transient in the vertical direction shear (wind veer).  This is a linear relationship between the height and the local direction, with the direction being its nominal value at the reference height.
    
    Attributes
    ----------
    DirectionShearVariationType : Literal['PresetTransient'], default='PresetTransient'
        Defines the specific type of DirectionShearVariation model in use.  For a `PresetTransient` object, this must always be set to a value of `PresetTransient`.
    
    DirectionShearAtStart : float
        The vertical direction shear at the start of the simulation up to the beginning of the transient.  For a full cycle this will also be the value after the transient has completed.
    
    AmplitudeOfVariation : float
        The maximum amplitude of the variation.
    
    StartTime : float
        The time into the simulation at which to start the transient (excluding the lead-in time).
    
    DurationOfVariation : float
        The time taken to complete the transient, whether half-cycle or full-cycle.
    
    VariationShape : PresetWindDirectionShearTransient_VariationShapeEnum
        The shape of the transient.  This can either be a half-cycle (where the shear remains at the initial value plus the amplitude after the transient has been completed) or a full-cycle (where the shear returns to its original value).
    
    Notes
    -----
    
    """
    DirectionShearVariationType: Literal['PresetTransient'] = Field(alias="DirectionShearVariationType", default='PresetTransient', allow_mutation=False, const=True) # type: ignore
    DirectionShearAtStart: float = Field(alias="DirectionShearAtStart", default=None)
    AmplitudeOfVariation: float = Field(alias="AmplitudeOfVariation", default=None)
    StartTime: float = Field(alias="StartTime", default=None)
    DurationOfVariation: float = Field(alias="DurationOfVariation", default=None)
    VariationShape: PresetWindDirectionShearTransient_VariationShapeEnum = Field(alias="VariationShape", default=None)

    _relative_schema_path = 'TimeDomainSimulation/Environment/Wind/WindDirectionShearVariation/PresetWindDirectionShearTransient.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'DirectionShearVariationType').merge(WindDirectionShearVariation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


PresetWindDirectionShearTransient.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.applied_load import AppliedLoad
from dnv_bladed_models.applied_load_insert import AppliedLoadInsert
from dnv_bladed_models.blade_point_loading import BladePointLoading
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.controller_fault import ControllerFault
from dnv_bladed_models.emergency_stop_operation import EmergencyStopOperation
from dnv_bladed_models.environment import Environment
from dnv_bladed_models.event import Event
from dnv_bladed_models.event_insert import EventInsert
from dnv_bladed_models.external_module import ExternalModule
from dnv_bladed_models.externally_stepped_simulation import ExternallySteppedSimulation
from dnv_bladed_models.gl2010_icing import GL2010Icing
from dnv_bladed_models.grid_loss import GridLoss
from dnv_bladed_models.iec4_icing import IEC4Icing
from dnv_bladed_models.initial_azimuth_position import InitialAzimuthPosition
from dnv_bladed_models.initial_condition import InitialCondition
from dnv_bladed_models.initial_condition_insert import InitialConditionInsert
from dnv_bladed_models.initial_floating_position import InitialFloatingPosition
from dnv_bladed_models.initial_pitch_position import InitialPitchPosition
from dnv_bladed_models.initial_rotor_speed import InitialRotorSpeed
from dnv_bladed_models.initial_yaw_angle import InitialYawAngle
from dnv_bladed_models.network_frequency_disturbance import NetworkFrequencyDisturbance
from dnv_bladed_models.network_voltage_disturbance import NetworkVoltageDisturbance
from dnv_bladed_models.normal_stop_operation import NormalStopOperation
from dnv_bladed_models.permanently_stuck_pitch_system import PermanentlyStuckPitchSystem
from dnv_bladed_models.pitch_fault_constant_rate import PitchFaultConstantRate
from dnv_bladed_models.pitch_fault_constant_torque import PitchFaultConstantTorque
from dnv_bladed_models.pitch_fault_limp import PitchFaultLimp
from dnv_bladed_models.pitch_fault_seizure import PitchFaultSeizure
from dnv_bladed_models.pitch_fault_seizure_at_angle import PitchFaultSeizureAtAngle
from dnv_bladed_models.rotor_idling_control_state import RotorIdlingControlState
from dnv_bladed_models.rotor_in_power_production import RotorInPowerProduction
from dnv_bladed_models.rotor_parked_control_state import RotorParkedControlState
from dnv_bladed_models.short_circuit import ShortCircuit
from dnv_bladed_models.start_up_operation import StartUpOperation
from dnv_bladed_models.time_domain_outputs import TimeDomainOutputs
from dnv_bladed_models.tower_point_loading import TowerPointLoading
from dnv_bladed_models.yaw_fault_constant_rate import YawFaultConstantRate
from dnv_bladed_models.yaw_fault_constant_torque import YawFaultConstantTorque
from dnv_bladed_models.yaw_fault_limp import YawFaultLimp
from dnv_bladed_models.yaw_manoeuvre import YawManoeuvre

from .schema_helper import SchemaHelper
from .models_impl import *

TAppliedLoadOptions = TypeVar('TAppliedLoadOptions', BladePointLoading, AppliedLoadInsert, TowerPointLoading, AppliedLoad, )
TInitialConditionOptions = TypeVar('TInitialConditionOptions', GL2010Icing, IEC4Icing, InitialAzimuthPosition, InitialFloatingPosition, InitialPitchPosition, InitialRotorSpeed, InitialYawAngle, InitialConditionInsert, RotorIdlingControlState, RotorInPowerProduction, RotorParkedControlState, InitialCondition, )
TEventOptions = TypeVar('TEventOptions', ControllerFault, EmergencyStopOperation, GridLoss, EventInsert, NetworkFrequencyDisturbance, NetworkVoltageDisturbance, NormalStopOperation, PermanentlyStuckPitchSystem, PitchFaultConstantRate, PitchFaultConstantTorque, PitchFaultLimp, PitchFaultSeizure, PitchFaultSeizureAtAngle, ShortCircuit, StartUpOperation, YawFaultConstantRate, YawFaultConstantTorque, YawFaultLimp, YawManoeuvre, Event, )

class TimeDomainSimulation(BladedModel):
    r"""
    The definition of a time domain analysis - mutually exclusive with 'SteadyCalculation'
    
    Attributes
    ----------
    Duration : float
        The duration of the simulation for which results will be output, excluding the LeadInTime. SI units are seconds.
    
    LeadInTime : float, default=7
        The time that the simulation will run before any outputs are recorded.  This is to allow the simulation to \"settle\", allowing any non-equilibrium initial conditions to converge on their steady state values.  The lead-in time will be added to the duration, meaning that the duration will be the length of time the outputs are recorded for.  Any times specified within the simulation will be measured from the *end* of the lead-in period.
    
    Outputs : TimeDomainOutputs
    
    AdditionalExternalModules : List[ExternalModule]
        A list of external module that will be used for this simulation only.  This could be used to apply loading to the structure, or some other load case specific purpose.  If the external module is seeking to simulate an intrinsic property of the turbine, consider moving it into the GlobalExternalModules, or adding it to the Assembly tree.
    
    ExternallySteppedSimulation : ExternallySteppedSimulation, Not supported yet
    
    Environment : Environment
    
    AppliedLoads : List[Union[BladePointLoading, AppliedLoadInsert, TowerPointLoading]]
        A list of point loading definitions which apply a time history of forces to the structure.
    
    InitialConditions : List[Union[GL2010Icing, IEC4Icing, InitialAzimuthPosition, InitialFloatingPosition, InitialPitchPosition, InitialRotorSpeed, InitialYawAngle, InitialConditionInsert, RotorIdlingControlState, RotorInPowerProduction, RotorParkedControlState]]
        A list of initial conditions to apply at the beginning of the simulation.
    
    Events : List[Union[ControllerFault, EmergencyStopOperation, GridLoss, EventInsert, NetworkFrequencyDisturbance, NetworkVoltageDisturbance, NormalStopOperation, PermanentlyStuckPitchSystem, PitchFaultConstantRate, PitchFaultConstantTorque, PitchFaultLimp, PitchFaultSeizure, PitchFaultSeizureAtAngle, ShortCircuit, StartUpOperation, YawFaultConstantRate, YawFaultConstantTorque, YawFaultLimp, YawManoeuvre]]
        A list of events that occur during the simulation.
    
    Notes
    -----
    
    """
    Duration: float = Field(alias="Duration", default=None)
    LeadInTime: float = Field(alias="LeadInTime", default=None)
    Outputs: TimeDomainOutputs = Field(alias="Outputs", default=None)
    AdditionalExternalModules: List[ExternalModule] = Field(alias="AdditionalExternalModules", default=list())
    ExternallySteppedSimulation: ExternallySteppedSimulation = Field(alias="ExternallySteppedSimulation", default=None) # Not supported yet
    Environment: Environment = Field(alias="Environment", default=None)
    AppliedLoads: List[Annotated[Union[BladePointLoading, AppliedLoadInsert, TowerPointLoading], Field(discriminator='AppliedLoadType')]] = Field(alias="AppliedLoads", default=list())
    InitialConditions: List[Annotated[Union[GL2010Icing, IEC4Icing, InitialAzimuthPosition, InitialFloatingPosition, InitialPitchPosition, InitialRotorSpeed, InitialYawAngle, InitialConditionInsert, RotorIdlingControlState, RotorInPowerProduction, RotorParkedControlState], Field(discriminator='InitialConditionType')]] = Field(alias="InitialConditions", default=list())
    Events: List[Annotated[Union[ControllerFault, EmergencyStopOperation, GridLoss, EventInsert, NetworkFrequencyDisturbance, NetworkVoltageDisturbance, NormalStopOperation, PermanentlyStuckPitchSystem, PitchFaultConstantRate, PitchFaultConstantTorque, PitchFaultLimp, PitchFaultSeizure, PitchFaultSeizureAtAngle, ShortCircuit, StartUpOperation, YawFaultConstantRate, YawFaultConstantTorque, YawFaultLimp, YawManoeuvre], Field(discriminator='EventType')]] = Field(alias="Events", default=list())

    _relative_schema_path = 'TimeDomainSimulation/TimeDomainSimulation.json'
    _type_info = TypeInfo(
        set([]),
        set([('AppliedLoads', 'AppliedLoadType'),('InitialConditions', 'InitialConditionType'),('Events', 'EventType'),]),
        set(['AdditionalExternalModules','AppliedLoads','InitialConditions','Events',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def AppliedLoads_as_inline(self) -> Iterable[Union[BladePointLoading, TowerPointLoading]]:
        """
        Retrieves the value of AppliedLoads as an iterable of model objects; if any of the values are specified with a '$insert', an error is raised.

        Returns
        -------
        Iterable[Union[BladePointLoading, TowerPointLoading]]
            A list of model objects, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If any of the values are not one of the concrete types of AppliedLoad; i.e. they are specified with an 'insert'.

        Examples
        --------
        >>> for i, AppliedLoad_obj in enumerate(obj.AppliedLoads_as_any):
                print(f"Element {i} is of type AppliedLoad_obj.AppliedLoadType}")

        or

        >>> for AppliedLoad_obj in obj.AppliedLoads_as_any:
                # process object
        """
        for val in self.AppliedLoads:
            if isinstance(val, AppliedLoadInsert) or val.is_insert == True:
                raise TypeError(f"Expected element value to be an in-line object, but it is currently in the '$insert' state.")
            yield val


    def AppliedLoads_element_as_BladePointLoading(self, index: int) -> BladePointLoading:
        """
        Retrieves an object from the AppliedLoads array field, ensuring it is a BladePointLoading.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        BladePointLoading
            A model object at the specified index, guaranteed to be a BladePointLoading.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a BladePointLoading.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a BladePointLoading, that is specified in-line:
        >>> entry_obj = obj.AppliedLoads_element_as_BladePointLoading(2)
        """
        return self.AppliedLoads_element_as(index, BladePointLoading)


    def AppliedLoads_element_as_TowerPointLoading(self, index: int) -> TowerPointLoading:
        """
        Retrieves an object from the AppliedLoads array field, ensuring it is a TowerPointLoading.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        TowerPointLoading
            A model object at the specified index, guaranteed to be a TowerPointLoading.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a TowerPointLoading.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a TowerPointLoading, that is specified in-line:
        >>> entry_obj = obj.AppliedLoads_element_as_TowerPointLoading(2)
        """
        return self.AppliedLoads_element_as(index, TowerPointLoading)


    def AppliedLoads_element_as_inline(self, index: int) -> Union[BladePointLoading, TowerPointLoading]:
        """
        Retrieves an object from the AppliedLoads array field; if the value is specified with a '$insert', an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Union[BladePointLoading, TowerPointLoading]
            A model object at the specified index, guaranteed to not be an '$insert'.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not one of the concrete types of AppliedLoad; i.e. it is specified with an '$insert'.

        Examples
        --------
        Get a reference to the 2nd element, as any of the concrete types:
        >>> AppliedLoads_obj = obj.AppliedLoads_element_as_inline(2)
        """
        val = self.AppliedLoads[index]
        if isinstance(val, AppliedLoadInsert) or val.is_insert == True:
            raise TypeError(f"Expected element value at '{index}' to be an in-line object, but it is currently in the '$insert' state.")
        return val


    def AppliedLoads_element_as(self, index: int, element_cls: Type[TAppliedLoadOptions]) -> TAppliedLoadOptions:
        """
        Retrieves an object from the AppliedLoads array field, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of AppliedLoad, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        element_cls: Type[Union[BladePointLoading, AppliedLoadInsert, TowerPointLoading]]
            One of the valid concrete types of AppliedLoad, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TAppliedLoadOptions
            A model object of the specified type.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the model object for the specified index is not of the specified type.

        Examples
        --------
        Get a reference to the 2nd element when it was one of the types of AppliedLoad:
        >>> entry_obj = obj.AppliedLoads_element_as(2, models.BladePointLoading)
        >>> entry_obj = obj.AppliedLoads_element_as(2, models.TowerPointLoading)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = obj.AppliedLoads_element_as(2, models.AppliedLoadInsert)
        """
        val = self.AppliedLoads[index]
        if not isinstance(val, element_cls):
            raise TypeError(f"Expected value of type '{element_cls.__name__}' at index {index} but found type '{type(val).__name__}'")
        return val


    @property
    def InitialConditions_as_inline(self) -> Iterable[Union[GL2010Icing, IEC4Icing, InitialAzimuthPosition, InitialFloatingPosition, InitialPitchPosition, InitialRotorSpeed, InitialYawAngle, RotorIdlingControlState, RotorInPowerProduction, RotorParkedControlState]]:
        """
        Retrieves the value of InitialConditions as an iterable of model objects; if any of the values are specified with a '$insert', an error is raised.

        Returns
        -------
        Iterable[Union[GL2010Icing, IEC4Icing, InitialAzimuthPosition, InitialFloatingPosition, InitialPitchPosition, InitialRotorSpeed, InitialYawAngle, RotorIdlingControlState, RotorInPowerProduction, RotorParkedControlState]]
            A list of model objects, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If any of the values are not one of the concrete types of InitialCondition; i.e. they are specified with an 'insert'.

        Examples
        --------
        >>> for i, InitialCondition_obj in enumerate(obj.InitialConditions_as_any):
                print(f"Element {i} is of type InitialCondition_obj.InitialConditionType}")

        or

        >>> for InitialCondition_obj in obj.InitialConditions_as_any:
                # process object
        """
        for val in self.InitialConditions:
            if isinstance(val, InitialConditionInsert) or val.is_insert == True:
                raise TypeError(f"Expected element value to be an in-line object, but it is currently in the '$insert' state.")
            yield val


    def InitialConditions_element_as_GL2010Icing(self, index: int) -> GL2010Icing:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a GL2010Icing.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        GL2010Icing
            A model object at the specified index, guaranteed to be a GL2010Icing.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a GL2010Icing.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a GL2010Icing, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_GL2010Icing(2)
        """
        return self.InitialConditions_element_as(index, GL2010Icing)


    def InitialConditions_element_as_IEC4Icing(self, index: int) -> IEC4Icing:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a IEC4Icing.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        IEC4Icing
            A model object at the specified index, guaranteed to be a IEC4Icing.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a IEC4Icing.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a IEC4Icing, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_IEC4Icing(2)
        """
        return self.InitialConditions_element_as(index, IEC4Icing)


    def InitialConditions_element_as_InitialAzimuthPosition(self, index: int) -> InitialAzimuthPosition:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a InitialAzimuthPosition.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        InitialAzimuthPosition
            A model object at the specified index, guaranteed to be a InitialAzimuthPosition.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a InitialAzimuthPosition.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a InitialAzimuthPosition, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_InitialAzimuthPosition(2)
        """
        return self.InitialConditions_element_as(index, InitialAzimuthPosition)


    def InitialConditions_element_as_InitialFloatingPosition(self, index: int) -> InitialFloatingPosition:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a InitialFloatingPosition.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        InitialFloatingPosition
            A model object at the specified index, guaranteed to be a InitialFloatingPosition.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a InitialFloatingPosition.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a InitialFloatingPosition, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_InitialFloatingPosition(2)
        """
        return self.InitialConditions_element_as(index, InitialFloatingPosition)


    def InitialConditions_element_as_InitialPitchPosition(self, index: int) -> InitialPitchPosition:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a InitialPitchPosition.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        InitialPitchPosition
            A model object at the specified index, guaranteed to be a InitialPitchPosition.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a InitialPitchPosition.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a InitialPitchPosition, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_InitialPitchPosition(2)
        """
        return self.InitialConditions_element_as(index, InitialPitchPosition)


    def InitialConditions_element_as_InitialRotorSpeed(self, index: int) -> InitialRotorSpeed:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a InitialRotorSpeed.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        InitialRotorSpeed
            A model object at the specified index, guaranteed to be a InitialRotorSpeed.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a InitialRotorSpeed.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a InitialRotorSpeed, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_InitialRotorSpeed(2)
        """
        return self.InitialConditions_element_as(index, InitialRotorSpeed)


    def InitialConditions_element_as_InitialYawAngle(self, index: int) -> InitialYawAngle:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a InitialYawAngle.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        InitialYawAngle
            A model object at the specified index, guaranteed to be a InitialYawAngle.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a InitialYawAngle.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a InitialYawAngle, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_InitialYawAngle(2)
        """
        return self.InitialConditions_element_as(index, InitialYawAngle)


    def InitialConditions_element_as_RotorIdlingControlState(self, index: int) -> RotorIdlingControlState:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a RotorIdlingControlState.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        RotorIdlingControlState
            A model object at the specified index, guaranteed to be a RotorIdlingControlState.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a RotorIdlingControlState.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a RotorIdlingControlState, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_RotorIdlingControlState(2)
        """
        return self.InitialConditions_element_as(index, RotorIdlingControlState)


    def InitialConditions_element_as_RotorInPowerProduction(self, index: int) -> RotorInPowerProduction:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a RotorInPowerProduction.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        RotorInPowerProduction
            A model object at the specified index, guaranteed to be a RotorInPowerProduction.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a RotorInPowerProduction.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a RotorInPowerProduction, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_RotorInPowerProduction(2)
        """
        return self.InitialConditions_element_as(index, RotorInPowerProduction)


    def InitialConditions_element_as_RotorParkedControlState(self, index: int) -> RotorParkedControlState:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is a RotorParkedControlState.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        RotorParkedControlState
            A model object at the specified index, guaranteed to be a RotorParkedControlState.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a RotorParkedControlState.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a RotorParkedControlState, that is specified in-line:
        >>> entry_obj = obj.InitialConditions_element_as_RotorParkedControlState(2)
        """
        return self.InitialConditions_element_as(index, RotorParkedControlState)


    def InitialConditions_element_as_inline(self, index: int) -> Union[GL2010Icing, IEC4Icing, InitialAzimuthPosition, InitialFloatingPosition, InitialPitchPosition, InitialRotorSpeed, InitialYawAngle, RotorIdlingControlState, RotorInPowerProduction, RotorParkedControlState]:
        """
        Retrieves an object from the InitialConditions array field; if the value is specified with a '$insert', an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Union[GL2010Icing, IEC4Icing, InitialAzimuthPosition, InitialFloatingPosition, InitialPitchPosition, InitialRotorSpeed, InitialYawAngle, RotorIdlingControlState, RotorInPowerProduction, RotorParkedControlState]
            A model object at the specified index, guaranteed to not be an '$insert'.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not one of the concrete types of InitialCondition; i.e. it is specified with an '$insert'.

        Examples
        --------
        Get a reference to the 2nd element, as any of the concrete types:
        >>> InitialConditions_obj = obj.InitialConditions_element_as_inline(2)
        """
        val = self.InitialConditions[index]
        if isinstance(val, InitialConditionInsert) or val.is_insert == True:
            raise TypeError(f"Expected element value at '{index}' to be an in-line object, but it is currently in the '$insert' state.")
        return val


    def InitialConditions_element_as(self, index: int, element_cls: Type[TInitialConditionOptions]) -> TInitialConditionOptions:
        """
        Retrieves an object from the InitialConditions array field, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of InitialCondition, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        element_cls: Type[Union[GL2010Icing, IEC4Icing, InitialAzimuthPosition, InitialFloatingPosition, InitialPitchPosition, InitialRotorSpeed, InitialYawAngle, InitialConditionInsert, RotorIdlingControlState, RotorInPowerProduction, RotorParkedControlState]]
            One of the valid concrete types of InitialCondition, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TInitialConditionOptions
            A model object of the specified type.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the model object for the specified index is not of the specified type.

        Examples
        --------
        Get a reference to the 2nd element when it was one of the types of InitialCondition:
        >>> entry_obj = obj.InitialConditions_element_as(2, models.GL2010Icing)
        >>> entry_obj = obj.InitialConditions_element_as(2, models.IEC4Icing)
        >>> entry_obj = obj.InitialConditions_element_as(2, models.InitialAzimuthPosition)
        >>> entry_obj = obj.InitialConditions_element_as(2, models.InitialFloatingPosition)
        >>> entry_obj = obj.InitialConditions_element_as(2, models.InitialPitchPosition)
        >>> entry_obj = obj.InitialConditions_element_as(2, models.InitialRotorSpeed)
        >>> entry_obj = obj.InitialConditions_element_as(2, models.InitialYawAngle)
        >>> entry_obj = obj.InitialConditions_element_as(2, models.RotorIdlingControlState)
        >>> entry_obj = obj.InitialConditions_element_as(2, models.RotorInPowerProduction)
        >>> entry_obj = obj.InitialConditions_element_as(2, models.RotorParkedControlState)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = obj.InitialConditions_element_as(2, models.InitialConditionInsert)
        """
        val = self.InitialConditions[index]
        if not isinstance(val, element_cls):
            raise TypeError(f"Expected value of type '{element_cls.__name__}' at index {index} but found type '{type(val).__name__}'")
        return val


    @property
    def Events_as_inline(self) -> Iterable[Union[ControllerFault, EmergencyStopOperation, GridLoss, NetworkFrequencyDisturbance, NetworkVoltageDisturbance, NormalStopOperation, PermanentlyStuckPitchSystem, PitchFaultConstantRate, PitchFaultConstantTorque, PitchFaultLimp, PitchFaultSeizure, PitchFaultSeizureAtAngle, ShortCircuit, StartUpOperation, YawFaultConstantRate, YawFaultConstantTorque, YawFaultLimp, YawManoeuvre]]:
        """
        Retrieves the value of Events as an iterable of model objects; if any of the values are specified with a '$insert', an error is raised.

        Returns
        -------
        Iterable[Union[ControllerFault, EmergencyStopOperation, GridLoss, NetworkFrequencyDisturbance, NetworkVoltageDisturbance, NormalStopOperation, PermanentlyStuckPitchSystem, PitchFaultConstantRate, PitchFaultConstantTorque, PitchFaultLimp, PitchFaultSeizure, PitchFaultSeizureAtAngle, ShortCircuit, StartUpOperation, YawFaultConstantRate, YawFaultConstantTorque, YawFaultLimp, YawManoeuvre]]
            A list of model objects, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If any of the values are not one of the concrete types of Event; i.e. they are specified with an 'insert'.

        Examples
        --------
        >>> for i, Event_obj in enumerate(obj.Events_as_any):
                print(f"Element {i} is of type Event_obj.EventType}")

        or

        >>> for Event_obj in obj.Events_as_any:
                # process object
        """
        for val in self.Events:
            if isinstance(val, EventInsert) or val.is_insert == True:
                raise TypeError(f"Expected element value to be an in-line object, but it is currently in the '$insert' state.")
            yield val


    def Events_element_as_ControllerFault(self, index: int) -> ControllerFault:
        """
        Retrieves an object from the Events array field, ensuring it is a ControllerFault.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        ControllerFault
            A model object at the specified index, guaranteed to be a ControllerFault.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a ControllerFault.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a ControllerFault, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_ControllerFault(2)
        """
        return self.Events_element_as(index, ControllerFault)


    def Events_element_as_EmergencyStopOperation(self, index: int) -> EmergencyStopOperation:
        """
        Retrieves an object from the Events array field, ensuring it is a EmergencyStopOperation.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        EmergencyStopOperation
            A model object at the specified index, guaranteed to be a EmergencyStopOperation.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a EmergencyStopOperation.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a EmergencyStopOperation, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_EmergencyStopOperation(2)
        """
        return self.Events_element_as(index, EmergencyStopOperation)


    def Events_element_as_GridLoss(self, index: int) -> GridLoss:
        """
        Retrieves an object from the Events array field, ensuring it is a GridLoss.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        GridLoss
            A model object at the specified index, guaranteed to be a GridLoss.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a GridLoss.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a GridLoss, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_GridLoss(2)
        """
        return self.Events_element_as(index, GridLoss)


    def Events_element_as_NetworkFrequencyDisturbance(self, index: int) -> NetworkFrequencyDisturbance:
        """
        Retrieves an object from the Events array field, ensuring it is a NetworkFrequencyDisturbance.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        NetworkFrequencyDisturbance
            A model object at the specified index, guaranteed to be a NetworkFrequencyDisturbance.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a NetworkFrequencyDisturbance.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a NetworkFrequencyDisturbance, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_NetworkFrequencyDisturbance(2)
        """
        return self.Events_element_as(index, NetworkFrequencyDisturbance)


    def Events_element_as_NetworkVoltageDisturbance(self, index: int) -> NetworkVoltageDisturbance:
        """
        Retrieves an object from the Events array field, ensuring it is a NetworkVoltageDisturbance.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        NetworkVoltageDisturbance
            A model object at the specified index, guaranteed to be a NetworkVoltageDisturbance.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a NetworkVoltageDisturbance.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a NetworkVoltageDisturbance, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_NetworkVoltageDisturbance(2)
        """
        return self.Events_element_as(index, NetworkVoltageDisturbance)


    def Events_element_as_NormalStopOperation(self, index: int) -> NormalStopOperation:
        """
        Retrieves an object from the Events array field, ensuring it is a NormalStopOperation.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        NormalStopOperation
            A model object at the specified index, guaranteed to be a NormalStopOperation.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a NormalStopOperation.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a NormalStopOperation, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_NormalStopOperation(2)
        """
        return self.Events_element_as(index, NormalStopOperation)


    def Events_element_as_PermanentlyStuckPitchSystem(self, index: int) -> PermanentlyStuckPitchSystem:
        """
        Retrieves an object from the Events array field, ensuring it is a PermanentlyStuckPitchSystem.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        PermanentlyStuckPitchSystem
            A model object at the specified index, guaranteed to be a PermanentlyStuckPitchSystem.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a PermanentlyStuckPitchSystem.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a PermanentlyStuckPitchSystem, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_PermanentlyStuckPitchSystem(2)
        """
        return self.Events_element_as(index, PermanentlyStuckPitchSystem)


    def Events_element_as_PitchFaultConstantRate(self, index: int) -> PitchFaultConstantRate:
        """
        Retrieves an object from the Events array field, ensuring it is a PitchFaultConstantRate.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        PitchFaultConstantRate
            A model object at the specified index, guaranteed to be a PitchFaultConstantRate.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a PitchFaultConstantRate.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a PitchFaultConstantRate, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_PitchFaultConstantRate(2)
        """
        return self.Events_element_as(index, PitchFaultConstantRate)


    def Events_element_as_PitchFaultConstantTorque(self, index: int) -> PitchFaultConstantTorque:
        """
        Retrieves an object from the Events array field, ensuring it is a PitchFaultConstantTorque.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        PitchFaultConstantTorque
            A model object at the specified index, guaranteed to be a PitchFaultConstantTorque.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a PitchFaultConstantTorque.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a PitchFaultConstantTorque, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_PitchFaultConstantTorque(2)
        """
        return self.Events_element_as(index, PitchFaultConstantTorque)


    def Events_element_as_PitchFaultLimp(self, index: int) -> PitchFaultLimp:
        """
        Retrieves an object from the Events array field, ensuring it is a PitchFaultLimp.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        PitchFaultLimp
            A model object at the specified index, guaranteed to be a PitchFaultLimp.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a PitchFaultLimp.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a PitchFaultLimp, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_PitchFaultLimp(2)
        """
        return self.Events_element_as(index, PitchFaultLimp)


    def Events_element_as_PitchFaultSeizure(self, index: int) -> PitchFaultSeizure:
        """
        Retrieves an object from the Events array field, ensuring it is a PitchFaultSeizure.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        PitchFaultSeizure
            A model object at the specified index, guaranteed to be a PitchFaultSeizure.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a PitchFaultSeizure.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a PitchFaultSeizure, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_PitchFaultSeizure(2)
        """
        return self.Events_element_as(index, PitchFaultSeizure)


    def Events_element_as_PitchFaultSeizureAtAngle(self, index: int) -> PitchFaultSeizureAtAngle:
        """
        Retrieves an object from the Events array field, ensuring it is a PitchFaultSeizureAtAngle.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        PitchFaultSeizureAtAngle
            A model object at the specified index, guaranteed to be a PitchFaultSeizureAtAngle.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a PitchFaultSeizureAtAngle.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a PitchFaultSeizureAtAngle, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_PitchFaultSeizureAtAngle(2)
        """
        return self.Events_element_as(index, PitchFaultSeizureAtAngle)


    def Events_element_as_ShortCircuit(self, index: int) -> ShortCircuit:
        """
        Retrieves an object from the Events array field, ensuring it is a ShortCircuit.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        ShortCircuit
            A model object at the specified index, guaranteed to be a ShortCircuit.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a ShortCircuit.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a ShortCircuit, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_ShortCircuit(2)
        """
        return self.Events_element_as(index, ShortCircuit)


    def Events_element_as_StartUpOperation(self, index: int) -> StartUpOperation:
        """
        Retrieves an object from the Events array field, ensuring it is a StartUpOperation.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        StartUpOperation
            A model object at the specified index, guaranteed to be a StartUpOperation.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a StartUpOperation.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a StartUpOperation, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_StartUpOperation(2)
        """
        return self.Events_element_as(index, StartUpOperation)


    def Events_element_as_YawFaultConstantRate(self, index: int) -> YawFaultConstantRate:
        """
        Retrieves an object from the Events array field, ensuring it is a YawFaultConstantRate.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        YawFaultConstantRate
            A model object at the specified index, guaranteed to be a YawFaultConstantRate.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a YawFaultConstantRate.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a YawFaultConstantRate, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_YawFaultConstantRate(2)
        """
        return self.Events_element_as(index, YawFaultConstantRate)


    def Events_element_as_YawFaultConstantTorque(self, index: int) -> YawFaultConstantTorque:
        """
        Retrieves an object from the Events array field, ensuring it is a YawFaultConstantTorque.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        YawFaultConstantTorque
            A model object at the specified index, guaranteed to be a YawFaultConstantTorque.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a YawFaultConstantTorque.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a YawFaultConstantTorque, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_YawFaultConstantTorque(2)
        """
        return self.Events_element_as(index, YawFaultConstantTorque)


    def Events_element_as_YawFaultLimp(self, index: int) -> YawFaultLimp:
        """
        Retrieves an object from the Events array field, ensuring it is a YawFaultLimp.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        YawFaultLimp
            A model object at the specified index, guaranteed to be a YawFaultLimp.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a YawFaultLimp.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a YawFaultLimp, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_YawFaultLimp(2)
        """
        return self.Events_element_as(index, YawFaultLimp)


    def Events_element_as_YawManoeuvre(self, index: int) -> YawManoeuvre:
        """
        Retrieves an object from the Events array field, ensuring it is a YawManoeuvre.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        YawManoeuvre
            A model object at the specified index, guaranteed to be a YawManoeuvre.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a YawManoeuvre.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a YawManoeuvre, that is specified in-line:
        >>> entry_obj = obj.Events_element_as_YawManoeuvre(2)
        """
        return self.Events_element_as(index, YawManoeuvre)


    def Events_element_as_inline(self, index: int) -> Union[ControllerFault, EmergencyStopOperation, GridLoss, NetworkFrequencyDisturbance, NetworkVoltageDisturbance, NormalStopOperation, PermanentlyStuckPitchSystem, PitchFaultConstantRate, PitchFaultConstantTorque, PitchFaultLimp, PitchFaultSeizure, PitchFaultSeizureAtAngle, ShortCircuit, StartUpOperation, YawFaultConstantRate, YawFaultConstantTorque, YawFaultLimp, YawManoeuvre]:
        """
        Retrieves an object from the Events array field; if the value is specified with a '$insert', an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Union[ControllerFault, EmergencyStopOperation, GridLoss, NetworkFrequencyDisturbance, NetworkVoltageDisturbance, NormalStopOperation, PermanentlyStuckPitchSystem, PitchFaultConstantRate, PitchFaultConstantTorque, PitchFaultLimp, PitchFaultSeizure, PitchFaultSeizureAtAngle, ShortCircuit, StartUpOperation, YawFaultConstantRate, YawFaultConstantTorque, YawFaultLimp, YawManoeuvre]
            A model object at the specified index, guaranteed to not be an '$insert'.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not one of the concrete types of Event; i.e. it is specified with an '$insert'.

        Examples
        --------
        Get a reference to the 2nd element, as any of the concrete types:
        >>> Events_obj = obj.Events_element_as_inline(2)
        """
        val = self.Events[index]
        if isinstance(val, EventInsert) or val.is_insert == True:
            raise TypeError(f"Expected element value at '{index}' to be an in-line object, but it is currently in the '$insert' state.")
        return val


    def Events_element_as(self, index: int, element_cls: Type[TEventOptions]) -> TEventOptions:
        """
        Retrieves an object from the Events array field, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of Event, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        element_cls: Type[Union[ControllerFault, EmergencyStopOperation, GridLoss, EventInsert, NetworkFrequencyDisturbance, NetworkVoltageDisturbance, NormalStopOperation, PermanentlyStuckPitchSystem, PitchFaultConstantRate, PitchFaultConstantTorque, PitchFaultLimp, PitchFaultSeizure, PitchFaultSeizureAtAngle, ShortCircuit, StartUpOperation, YawFaultConstantRate, YawFaultConstantTorque, YawFaultLimp, YawManoeuvre]]
            One of the valid concrete types of Event, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TEventOptions
            A model object of the specified type.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the model object for the specified index is not of the specified type.

        Examples
        --------
        Get a reference to the 2nd element when it was one of the types of Event:
        >>> entry_obj = obj.Events_element_as(2, models.ControllerFault)
        >>> entry_obj = obj.Events_element_as(2, models.EmergencyStopOperation)
        >>> entry_obj = obj.Events_element_as(2, models.GridLoss)
        >>> entry_obj = obj.Events_element_as(2, models.NetworkFrequencyDisturbance)
        >>> entry_obj = obj.Events_element_as(2, models.NetworkVoltageDisturbance)
        >>> entry_obj = obj.Events_element_as(2, models.NormalStopOperation)
        >>> entry_obj = obj.Events_element_as(2, models.PermanentlyStuckPitchSystem)
        >>> entry_obj = obj.Events_element_as(2, models.PitchFaultConstantRate)
        >>> entry_obj = obj.Events_element_as(2, models.PitchFaultConstantTorque)
        >>> entry_obj = obj.Events_element_as(2, models.PitchFaultLimp)
        >>> entry_obj = obj.Events_element_as(2, models.PitchFaultSeizure)
        >>> entry_obj = obj.Events_element_as(2, models.PitchFaultSeizureAtAngle)
        >>> entry_obj = obj.Events_element_as(2, models.ShortCircuit)
        >>> entry_obj = obj.Events_element_as(2, models.StartUpOperation)
        >>> entry_obj = obj.Events_element_as(2, models.YawFaultConstantRate)
        >>> entry_obj = obj.Events_element_as(2, models.YawFaultConstantTorque)
        >>> entry_obj = obj.Events_element_as(2, models.YawFaultLimp)
        >>> entry_obj = obj.Events_element_as(2, models.YawManoeuvre)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = obj.Events_element_as(2, models.EventInsert)
        """
        val = self.Events[index]
        if not isinstance(val, element_cls):
            raise TypeError(f"Expected value of type '{element_cls.__name__}' at index {index} but found type '{type(val).__name__}'")
        return val


    def _entity(self) -> bool:
        return True


TimeDomainSimulation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.equilibrium_state_solver_settings import EquilibriumStateSolverSettings
from dnv_bladed_models.explicit_newmark_beta_fixed_step import ExplicitNewmarkBetaFixedStep
from dnv_bladed_models.generalised_alpha_fixed_step import GeneralisedAlphaFixedStep
from dnv_bladed_models.implicit_newmark_beta_fixed_step import ImplicitNewmarkBetaFixedStep
from dnv_bladed_models.integrator import Integrator
from dnv_bladed_models.integrator_insert import IntegratorInsert
from dnv_bladed_models.runge_kutta_variable_step import RungeKuttaVariableStep

from .schema_helper import SchemaHelper
from .models_impl import *

TIntegratorOptions = TypeVar('TIntegratorOptions', ExplicitNewmarkBetaFixedStep, GeneralisedAlphaFixedStep, ImplicitNewmarkBetaFixedStep, IntegratorInsert, RungeKuttaVariableStep, Integrator, )

class SolverSettings(BladedModel):
    r"""
    Settings for the numerical solving of the analysis, including integrator settings.
    
    Attributes
    ----------
    EquilibriumStateIteration : EquilibriumStateSolverSettings
    
    TimeDomainIntegration : Union[ExplicitNewmarkBetaFixedStep, GeneralisedAlphaFixedStep, ImplicitNewmarkBetaFixedStep, IntegratorInsert, RungeKuttaVariableStep]
    
    Notes
    -----
    
    """
    EquilibriumStateIteration: EquilibriumStateSolverSettings = Field(alias="EquilibriumStateIteration", default=None)
    TimeDomainIntegration: Union[ExplicitNewmarkBetaFixedStep, GeneralisedAlphaFixedStep, ImplicitNewmarkBetaFixedStep, IntegratorInsert, RungeKuttaVariableStep] = Field(alias="TimeDomainIntegration", default=None, discriminator='IntegratorType')

    _relative_schema_path = 'Settings/SolverSettings/SolverSettings.json'
    _type_info = TypeInfo(
        set([('TimeDomainIntegration', 'IntegratorType'),]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def TimeDomainIntegration_as_ExplicitNewmarkBetaFixedStep(self) -> ExplicitNewmarkBetaFixedStep:
        """
        Retrieves the value of TimeDomainIntegration guaranteeing it is a ExplicitNewmarkBetaFixedStep; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ExplicitNewmarkBetaFixedStep
            A model object, guaranteed to be a ExplicitNewmarkBetaFixedStep.

        Raises
        ------
        TypeError
            If the value is not a ExplicitNewmarkBetaFixedStep.
        """
        return self.TimeDomainIntegration_as(ExplicitNewmarkBetaFixedStep)


    @property
    def TimeDomainIntegration_as_GeneralisedAlphaFixedStep(self) -> GeneralisedAlphaFixedStep:
        """
        Retrieves the value of TimeDomainIntegration guaranteeing it is a GeneralisedAlphaFixedStep; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        GeneralisedAlphaFixedStep
            A model object, guaranteed to be a GeneralisedAlphaFixedStep.

        Raises
        ------
        TypeError
            If the value is not a GeneralisedAlphaFixedStep.
        """
        return self.TimeDomainIntegration_as(GeneralisedAlphaFixedStep)


    @property
    def TimeDomainIntegration_as_ImplicitNewmarkBetaFixedStep(self) -> ImplicitNewmarkBetaFixedStep:
        """
        Retrieves the value of TimeDomainIntegration guaranteeing it is a ImplicitNewmarkBetaFixedStep; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ImplicitNewmarkBetaFixedStep
            A model object, guaranteed to be a ImplicitNewmarkBetaFixedStep.

        Raises
        ------
        TypeError
            If the value is not a ImplicitNewmarkBetaFixedStep.
        """
        return self.TimeDomainIntegration_as(ImplicitNewmarkBetaFixedStep)


    @property
    def TimeDomainIntegration_as_RungeKuttaVariableStep(self) -> RungeKuttaVariableStep:
        """
        Retrieves the value of TimeDomainIntegration guaranteeing it is a RungeKuttaVariableStep; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        RungeKuttaVariableStep
            A model object, guaranteed to be a RungeKuttaVariableStep.

        Raises
        ------
        TypeError
            If the value is not a RungeKuttaVariableStep.
        """
        return self.TimeDomainIntegration_as(RungeKuttaVariableStep)


    @property
    def TimeDomainIntegration_as_inline(self) -> Union[ExplicitNewmarkBetaFixedStep, GeneralisedAlphaFixedStep, ImplicitNewmarkBetaFixedStep, RungeKuttaVariableStep]:
        """
        Retrieves the value of TimeDomainIntegration as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[ExplicitNewmarkBetaFixedStep, GeneralisedAlphaFixedStep, ImplicitNewmarkBetaFixedStep, RungeKuttaVariableStep]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of Integrator; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.TimeDomainIntegration, IntegratorInsert) or self.TimeDomainIntegration.is_insert:
            raise TypeError(f"Expected TimeDomainIntegration value to be an in-line object, but it is currently in the '$insert' state.")
        return self.TimeDomainIntegration


    def TimeDomainIntegration_as(self, cls: Type[TIntegratorOptions])-> TIntegratorOptions:
        """
        Retrieves the value of TimeDomainIntegration, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of Integrator, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[ExplicitNewmarkBetaFixedStep, GeneralisedAlphaFixedStep, ImplicitNewmarkBetaFixedStep, IntegratorInsert, RungeKuttaVariableStep]]
            One of the valid concrete types of Integrator, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TIntegratorOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of Integrator:
        >>> val_obj = model_obj.TimeDomainIntegration_as(models.ExplicitNewmarkBetaFixedStep)
        >>> val_obj = model_obj.TimeDomainIntegration_as(models.GeneralisedAlphaFixedStep)
        >>> val_obj = model_obj.TimeDomainIntegration_as(models.ImplicitNewmarkBetaFixedStep)
        >>> val_obj = model_obj.TimeDomainIntegration_as(models.RungeKuttaVariableStep)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.TimeDomainIntegration_as(models.IntegratorInsert)
        """
        if not isinstance(self.TimeDomainIntegration, cls):
            raise TypeError(f"Expected TimeDomainIntegration of type '{cls.__name__}' but was type '{type(self.TimeDomainIntegration).__name__}'")
        return self.TimeDomainIntegration


    def _entity(self) -> bool:
        return True


SolverSettings.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.component import Component
from dnv_bladed_models.extension_pieces import ExtensionPieces
from dnv_bladed_models.hub_connectable_nodes import HubConnectableNodes
from dnv_bladed_models.hub_mass_properties import HubMassProperties
from dnv_bladed_models.hub_output_group_library import HubOutputGroupLibrary
from dnv_bladed_models.mounting_points import MountingPoints
from dnv_bladed_models.spinner import Spinner
class Hub_RotationDirectionEnum(str, Enum):
    CLOCKWISE_VIEWED_FROM_FRONT = "CLOCKWISE_VIEWED_FROM_FRONT"
    ANTICLOCKWISE_VIEWED_FROM_FRONT = "ANTICLOCKWISE_VIEWED_FROM_FRONT"
class Hub_UpwindOrDownwindEnum(str, Enum):
    UPWIND = "UPWIND"
    DOWNWIND = "DOWNWIND"

from .schema_helper import SchemaHelper
from .models_impl import *


class Hub(Component, ABC):
    r"""
    The common properties shared by all hubs.
    
    Attributes
    ----------
    RotationDirection : Hub_RotationDirectionEnum
        The rotational sense of rotor when viewed from upwind.
    
    UpwindOrDownwind : Hub_UpwindOrDownwindEnum
        Whether the turbine is an upwind turbine (where the rotor is upwind of the nacelle) or a downwind turbine (where the rotor is downwind of the nacelle).
    
    MountingPoints : MountingPoints
    
    ExtensionPieces : ExtensionPieces
    
    MassProperties : HubMassProperties
    
    Spinner : Spinner
    
    ConnectableNodes : HubConnectableNodes, Not supported yet
    
    OutputGroups : HubOutputGroupLibrary, Not supported yet
    
    Notes
    -----
    
    """
    RotationDirection: Hub_RotationDirectionEnum = Field(alias="RotationDirection", default=None)
    UpwindOrDownwind: Hub_UpwindOrDownwindEnum = Field(alias="UpwindOrDownwind", default=None)
    MountingPoints: MountingPoints = Field(alias="MountingPoints", default=None)
    ExtensionPieces: ExtensionPieces = Field(alias="ExtensionPieces", default=None)
    MassProperties: HubMassProperties = Field(alias="MassProperties", default=None)
    Spinner: Spinner = Field(alias="Spinner", default=None)
    ConnectableNodes: HubConnectableNodes = Field(alias="ConnectableNodes", default=HubConnectableNodes()) # Not supported yet
    OutputGroups: HubOutputGroupLibrary = Field(alias="OutputGroups", default=HubOutputGroupLibrary()) # Not supported yet

    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['ConnectableNodes','OutputGroups',]),
        None).merge(Component._type_info)


Hub.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.shaft_input_power_vs_power_loss import ShaftInputPowerVsPowerLoss
from dnv_bladed_models.shaft_input_torque_vs_resisting_torque import ShaftInputTorqueVsResistingTorque
from dnv_bladed_models.shaft_speed_vs_shaft_input_power_vs_power_loss import ShaftSpeedVsShaftInputPowerVsPowerLoss
from dnv_bladed_models.shaft_speed_vs_shaft_input_torque_vs_resisting_torque import ShaftSpeedVsShaftInputTorqueVsResistingTorque

from .schema_helper import SchemaHelper
from .models_impl import *


class MechanicalLosses(BladedModel):
    r"""
    The common properties for the mechanical losses in the drivetrain.
    
    Attributes
    ----------
    ShaftSpeedVsShaftInputTorqueVsResistingTorque : List[ShaftSpeedVsShaftInputTorqueVsResistingTorque]
        A series of look-up tables for the losses, each valid for the specified shaft speed.
    
    ShaftInputTorqueVsResistingTorque : List[ShaftInputTorqueVsResistingTorque]
        A look-up table for the losses, each valid for the specified input torque.
    
    ShaftSpeedVsShaftInputPowerVsPowerLoss : List[ShaftSpeedVsShaftInputPowerVsPowerLoss]
        A series of look-up tables for the losses, each valid for the specified shaft rotational speed.
    
    ShaftInputPowerVsPowerLoss : List[ShaftInputPowerVsPowerLoss]
        A series of look-up tables for the losses, each valid for the specified shaft rotational speed.
    
    Notes
    -----
    
    """
    ShaftSpeedVsShaftInputTorqueVsResistingTorque: List[ShaftSpeedVsShaftInputTorqueVsResistingTorque] = Field(alias="ShaftSpeedVsShaftInputTorqueVsResistingTorque", default=list())
    ShaftInputTorqueVsResistingTorque: List[ShaftInputTorqueVsResistingTorque] = Field(alias="ShaftInputTorqueVsResistingTorque", default=list())
    ShaftSpeedVsShaftInputPowerVsPowerLoss: List[ShaftSpeedVsShaftInputPowerVsPowerLoss] = Field(alias="ShaftSpeedVsShaftInputPowerVsPowerLoss", default=list())
    ShaftInputPowerVsPowerLoss: List[ShaftInputPowerVsPowerLoss] = Field(alias="ShaftInputPowerVsPowerLoss", default=list())

    _relative_schema_path = 'Components/DrivetrainAndNacelle/MechanicalLosses/MechanicalLosses.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['ShaftSpeedVsShaftInputTorqueVsResistingTorque','ShaftInputTorqueVsResistingTorque','ShaftSpeedVsShaftInputPowerVsPowerLoss','ShaftInputPowerVsPowerLoss',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


MechanicalLosses.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.dynamic_stall import DynamicStall

from .schema_helper import SchemaHelper
from .models_impl import *


class BeddoesLeishmanModel(DynamicStall, ABC):
    r"""
    The common properties for the Beddoes Leishman dynamic stall models.
    
    Attributes
    ----------
    UseKirchoffFlow : bool, default=False
        If true,the normal force coefficient is computed using the dynamic separation position in Kirchoff's equation directly.  If false, the dynamic separation position is used to linearly interpolate between fully separated and fully attached flow. In normal operating conditions this setting will not lead to significant differences, but it has been found that in parked/idling cases (where the blade experiences high angles of attack) this option will improve the aerodynamic damping of the blade. The explanation for the damping differences is given in the aerodynamic validation document on the User Portal.
    
    UseImpulsiveContributions : bool, default=False
        Enabling the impulsive contributions in lift and moment coefficient.
    
    PressureLagTimeConstant : float, default=1.7
        The lag between the pressure and the lift.
    
    VortexLiftTimeConstant : float, default=6
        The rate of decay of the vortex induced lift that is generated when aerofoils undergo a rapid change in angle of attack towards positive or negative stall.
    
    VortexTravelTimeConstant : float, default=7.5
        The time constant that controls the duration of the vortex induced lift that is generated when aerofoils undergo a rapid change in angle of attack towards positive or negative stall.
    
    AttachedFlowConstantA1 : float, default=0.165
        The constant A1 for the Beddoes-Leishman type dynamic stall model to control the attached flow states.
    
    AttachedFlowConstantA2 : float, default=0.335
        The constant A2 for the Beddoes-Leishman type dynamic stall model to control the attached flow states.
    
    AttachedFlowConstantB1 : float, default=0.0455
        The constant B1 for the Beddoes-Leishman type dynamic stall model to control the attached flow states.
    
    AttachedFlowConstantB2 : float, default=0.3
        The constant B2 for the Beddoes-Leishman type dynamic stall model to control the attached flow states.
    
    Notes
    -----
    
    """
    UseKirchoffFlow: bool = Field(alias="UseKirchoffFlow", default=None)
    UseImpulsiveContributions: bool = Field(alias="UseImpulsiveContributions", default=None)
    PressureLagTimeConstant: float = Field(alias="PressureLagTimeConstant", default=None)
    VortexLiftTimeConstant: float = Field(alias="VortexLiftTimeConstant", default=None)
    VortexTravelTimeConstant: float = Field(alias="VortexTravelTimeConstant", default=None)
    AttachedFlowConstantA1: float = Field(alias="AttachedFlowConstantA1", default=None)
    AttachedFlowConstantA2: float = Field(alias="AttachedFlowConstantA2", default=None)
    AttachedFlowConstantB1: float = Field(alias="AttachedFlowConstantB1", default=None)
    AttachedFlowConstantB2: float = Field(alias="AttachedFlowConstantB2", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(DynamicStall._type_info)


BeddoesLeishmanModel.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.assembly import Assembly
from dnv_bladed_models.bladed_control import BladedControl
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.component_library import ComponentLibrary
from dnv_bladed_models.electrical_grid import ElectricalGrid
from dnv_bladed_models.external_module import ExternalModule
from dnv_bladed_models.turbine_operational_parameters import TurbineOperationalParameters

from .schema_helper import SchemaHelper
from .models_impl import *


class Turbine(BladedModel):
    r"""
    The definition of the turbine and its installation that is to be modelled.
    
    Attributes
    ----------
    ElectricalGrid : ElectricalGrid, Not supported yet
    
    TurbineOperationalParameters : TurbineOperationalParameters
    
    Control : BladedControl
    
    GlobalExternalModules : List[ExternalModule], Not supported yet
        A list of any external modules that will be run with the time domain simulations.  It is expected that external modules defined here will interact with more than one area of the turbine, such as to apply additional aerodynamics loads to the entire structure.  Any external modules that represent a single component should be added to the Assembly tree.
    
    MeanSeaLevel : float, default=0, Not supported yet
        The mean sea depth at the turbine location.  If omited, the Turbine will be considered an on-shore turbine and any sea states will be ignored.
    
    Assembly : Assembly
    
    ComponentLibrary : ComponentLibrary
    
    Notes
    -----
    
    """
    ElectricalGrid: ElectricalGrid = Field(alias="ElectricalGrid", default=None) # Not supported yet
    TurbineOperationalParameters: TurbineOperationalParameters = Field(alias="TurbineOperationalParameters", default=None)
    Control: BladedControl = Field(alias="Control", default=None)
    GlobalExternalModules: List[ExternalModule] = Field(alias="GlobalExternalModules", default=list()) # Not supported yet
    MeanSeaLevel: float = Field(alias="MeanSeaLevel", default=None) # Not supported yet
    Assembly: Assembly = Field(alias="Assembly", default=Assembly())
    ComponentLibrary: ComponentLibrary = Field(alias="ComponentLibrary", default=ComponentLibrary())

    _relative_schema_path = 'Turbine/Turbine.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['GlobalExternalModules','Assembly','ComponentLibrary',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


Turbine.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.aerodynamic_model import AerodynamicModel
class VortexLine_WakeTypeEnum(str, Enum):
    FREE_WAKE = "FreeWake"
    FIXED_WAKE = "FixedWake"
class VortexLine_CoreGrowthModelEnum(str, Enum):
    RL_MODEL = "RL_Model"
    LO_MODEL = "LO_Model"
    FIXED = "Fixed"
class VortexLine_InitialVortexCoreSizeModelEnum(str, Enum):
    RL_MODEL = "RL_Model"
    LO_MODEL = "LO_Model"
    FIXED = "Fixed"

from .schema_helper import SchemaHelper
from .models_impl import *


class VortexLine(AerodynamicModel):
    r"""
    The Vortex Line aerodynamic model.
    
    Not supported yet.
    
    Attributes
    ----------
    AerodynamicModelType : Literal['VortexLine'], default='VortexLine', Not supported yet
        Defines the specific type of AerodynamicModel model in use.  For a `VortexLine` object, this must always be set to a value of `VortexLine`.
    
    MaximumNumberofFreeWakeSteps : int, default=200, Not supported yet
        Each free wake node that is emitted from the trailing edge will be allowed a maximum number of free wake steps after it will be no longer considered in the free wake solution and convected with local wind speed and last computed induction.
    
    MaximumNumberofWakeSteps : int, default=10000, Not supported yet
        Each wake node will be allowed a maximum number of steps before it is removed. This option puts an upper bound on the number of wake nodes.
    
    NumberOfThreads : int, default=1, Not supported yet
        The number of parallel CPU threads used in evaluation of the Biot-Savart law.  This option is only relevant when the wake type is set to \"Free Wake\".
    
    VortexWakeTimeStep : float, default=0.05, Not supported yet
        The time step used to update the vortex wake.  It is recommended to select a time step such that at least 60 vortex wake steps are taken each rotor revolution.
    
    WakeType : VortexLine_WakeTypeEnum, default='FreeWake', Not supported yet
        The \"Free Wake\" option will calculate the mutual influence of all wake elements on all wake nodes during each time step.  The \"Fixed Wake\" option will assume that the induced velocity in all wake nodes is equal to the average wake induced velocity at 70% blade radius.  The \"Free Wake\" option requires substantially more calculations to be performed, and is likely to significantly slow the analysis.
    
    CoreGrowthModel : VortexLine_CoreGrowthModelEnum, default='RL_Model', Not supported yet
        The Core Growth Model.
    
    InitialVortexCoreSizeModel : VortexLine_InitialVortexCoreSizeModelEnum, default='RL_Model', Not supported yet
        The intial vortex core size Model.
    
    FilamentStrain : bool, default=True, Not supported yet
        The filament strain.
    
    LambOseenCoreGrowthConstant : float, default=1.234, Not supported yet
        The Lamb-Oseen core growth constant,
    
    CoreGrowthConstant : float, default=50, Not supported yet
        The core growth constant.
    
    RamasamyLeishmanConstant : float, default=0.000065, Not supported yet
        The Ramasamy-Leishman constant.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    AerodynamicModelType: Literal['VortexLine'] = Field(alias="AerodynamicModelType", default='VortexLine', allow_mutation=False, const=True) # Not supported yet # type: ignore
    MaximumNumberofFreeWakeSteps: int = Field(alias="MaximumNumberofFreeWakeSteps", default=None) # Not supported yet
    MaximumNumberofWakeSteps: int = Field(alias="MaximumNumberofWakeSteps", default=None) # Not supported yet
    NumberOfThreads: int = Field(alias="NumberOfThreads", default=None) # Not supported yet
    VortexWakeTimeStep: float = Field(alias="VortexWakeTimeStep", default=None) # Not supported yet
    WakeType: VortexLine_WakeTypeEnum = Field(alias="WakeType", default=None) # Not supported yet
    CoreGrowthModel: VortexLine_CoreGrowthModelEnum = Field(alias="CoreGrowthModel", default=None) # Not supported yet
    InitialVortexCoreSizeModel: VortexLine_InitialVortexCoreSizeModelEnum = Field(alias="InitialVortexCoreSizeModel", default=None) # Not supported yet
    FilamentStrain: bool = Field(alias="FilamentStrain", default=None) # Not supported yet
    LambOseenCoreGrowthConstant: float = Field(alias="LambOseenCoreGrowthConstant", default=None) # Not supported yet
    CoreGrowthConstant: float = Field(alias="CoreGrowthConstant", default=None) # Not supported yet
    RamasamyLeishmanConstant: float = Field(alias="RamasamyLeishmanConstant", default=None) # Not supported yet

    _relative_schema_path = 'Settings/AerodynamicSettings/AerodynamicModel/VortexLine.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'AerodynamicModelType').merge(AerodynamicModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


VortexLine.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.damper import Damper
from dnv_bladed_models.vector3_d import Vector3D

from .schema_helper import SchemaHelper
from .models_impl import *


class LinearPassiveDamper(Damper):
    r"""
    A linear vibration damper to add to the support structure.
    
    Not supported yet.
    
    Attributes
    ----------
    ComponentType : Literal['LinearPassiveDamper'], default='LinearPassiveDamper', Not supported yet
        Defines the specific type of Component model in use.  For a `LinearPassiveDamper` object, this must always be set to a value of `LinearPassiveDamper`.
    
    Mass : float, Not supported yet
        The active mass of the vibration damper.
    
    Frequency : float, Not supported yet
        The peak frequency which the vibration damper responds to.
    
    Damping : float, Not supported yet
        The fraction of critical damping that the vibration damper will achive.
    
    IgnoreInModalAnalysis : bool, Not supported yet
        If true, the damper will be ignored in modal analyses.
    
    Direction : Vector3D
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    ComponentType: Literal['LinearPassiveDamper'] = Field(alias="ComponentType", default='LinearPassiveDamper', allow_mutation=False, const=True) # Not supported yet # type: ignore
    Mass: float = Field(alias="Mass", default=None) # Not supported yet
    Frequency: float = Field(alias="Frequency", default=None) # Not supported yet
    Damping: float = Field(alias="Damping", default=None) # Not supported yet
    IgnoreInModalAnalysis: bool = Field(alias="IgnoreInModalAnalysis", default=None) # Not supported yet
    Direction: Vector3D = Field(alias="Direction", default=None)

    _relative_schema_path = 'Components/Damper/LinearPassiveDamper.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'ComponentType').merge(Damper._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


LinearPassiveDamper.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class Component_ComponentTypeEnum(str, Enum):
    BLADE = "Blade"
    DRIVETRAIN_AND_NACELLE = "DrivetrainAndNacelle"
    EXTERNAL_MODULE_COMPONENT = "ExternalModuleComponent"
    FIXED_SPEED_ACTIVE_DAMPER = "FixedSpeedActiveDamper"
    FLEXIBILITY = "Flexibility"
    INDEPENDENT_PITCH_HUB = "IndependentPitchHub"
    LIDAR = "Lidar"
    LINEAR_PASSIVE_DAMPER = "LinearPassiveDamper"
    PENDULUM_DAMPER = "PendulumDamper"
    PITCH_SYSTEM = "PitchSystem"
    RIGID_BODY_POINT_INERTIA = "RigidBodyPointInertia"
    RIGID_BODY_SIXBY_SIX_INERTIA = "RigidBodySixbySixInertia"
    ROTATION = "Rotation"
    SUPERELEMENT = "Superelement"
    TOWER = "Tower"
    TRANSLATION = "Translation"
    VARIABLE_SPEED_ACTIVE_DAMPER = "VariableSpeedActiveDamper"
    VARIABLE_SPEED_GENERATOR = "VariableSpeedGenerator"
    YAW_SYSTEM = "YawSystem"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *


class Component(BladedModel, ABC):
    r"""
    The common properties of all components.
    
    Attributes
    ----------
    ComponentType : Component_ComponentTypeEnum
        Defines the specific type of model in use.
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - Blade
        - DrivetrainAndNacelle
        - ExternalModuleComponent
        - FixedSpeedActiveDamper
        - Flexibility
        - IndependentPitchHub
        - ComponentInsert
        - Lidar
        - LinearPassiveDamper
        - PendulumDamper
        - PitchSystem
        - RigidBodyPointInertia
        - RigidBodySixbySixInertia
        - Rotation
        - Superelement
        - Tower
        - Translation
        - VariableSpeedActiveDamper
        - VariableSpeedGenerator
        - YawSystem
    
    """
    ComponentType: Component_ComponentTypeEnum = Field(alias="ComponentType", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


Component.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.integrator import Integrator

from .schema_helper import SchemaHelper
from .models_impl import *


class FixedStepIntegrator(Integrator, ABC):
    r"""
    Common settings for the fixed step integrators.
    
    Attributes
    ----------
    TimeStep : float
        The fixed time step used by the integrator.  It must be set as a divisor of the output time-step and external controller communication interval.
    
    Tolerance : float, default=0.005
        When the \"Maximum number of iterations\" > 1, the integrator relative tolerance is used to control how many iterations are carried out when integrating the first order and prescribed second order states.  Iterations are carried out until the maximum number of iterations is reached, or until the change in all first order and prescribed state derivatives between successive iterations is less than the relative tolerance multiplied by the state derivative absolute value.
    
    Notes
    -----
    
    """
    TimeStep: float = Field(alias="TimeStep", default=None)
    Tolerance: float = Field(alias="Tolerance", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(Integrator._type_info)


FixedStepIntegrator.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.alpha_line import AlphaLine
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class Aerofoil(BladedModel):
    r"""
    The definition of a single 2D aerodynamic aerofoil section.
    
    Attributes
    ----------
    ChordwiseOriginForForcesAndMoments : float, default=0.25
        The fraction of the chord from the leading edge to the position about which Cl, Cd, and Cm have been calculated.  This is traditionally 0.25 (25%), but can be any location so long as it is consistent with the provided aerodynamic data.
    
    AerodynamicPerformance : List[AlphaLine]
        The relationship between angle of attack, and the lift and drag coefficients.
    
    ReynoldsNumber : float
        The Reynolds number at which the aerodynamic properties were obtained.
    
    ThicknessToChordRatio : float
        The ratio between the thickness and the chord length of the measured aerofoil.
    
    Notes
    -----
    
    """
    ChordwiseOriginForForcesAndMoments: float = Field(alias="ChordwiseOriginForForcesAndMoments", default=None)
    AerodynamicPerformance: List[AlphaLine] = Field(alias="AerodynamicPerformance", default=list())
    ReynoldsNumber: float = Field(alias="ReynoldsNumber", default=None)
    ThicknessToChordRatio: float = Field(alias="ThicknessToChordRatio", default=None)

    _relative_schema_path = 'Components/Blade/AerofoilLibrary/Aerofoil/Aerofoil.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['AerodynamicPerformance',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


Aerofoil.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class BladeModelling_BladeModellingTypeEnum(str, Enum):
    FINITE_ELEMENT_BLADE_MODELLING = "FiniteElementBladeModelling"
    MODAL_BLADE_MODELLING = "ModalBladeModelling"
    RIGID_BLADE_MODELLING = "RigidBladeModelling"
    INSERT = "Insert"
class BladeModelling_GeometricStiffnessModelEnum(str, Enum):
    AXIAL_LOADS_ONLY = "AxialLoadsOnly"
    FULL_MODEL_WITH_ORIENTATION_CORRECTION = "FullModelWithOrientationCorrection"
    INTERNAL_LOADS_ONLY = "InternalLoadsOnly"
    DISABLED = "Disabled"

from .schema_helper import SchemaHelper
from .models_impl import *


class BladeModelling(BladedModel, ABC):
    r"""
    Common properties for all blade modelling methods.
    
    Attributes
    ----------
    BladeModellingType : BladeModelling_BladeModellingTypeEnum
        Defines the specific type of model in use.
    
    GeometricStiffnessModel : BladeModelling_GeometricStiffnessModelEnum, default='AxialLoadsOnly'
        The geometric stiffness model to use for the blades. For blades with 1 part, the \"axial loads only\" model is recommended. This configuration is only appropriate for relatively stiff blades, undergoing small deflection.  For more flexible blade models, a multi-part blade model is more appropriate. In this case, the \"full with orientation correction\" is the recommended option, as long as deflection remains small within each blade part.
    
    IgnoreAxesOrientationDifferencesForShear : bool, default=False
        With this option selected, the effect of orientation difference between the elastic axis and shear axis on the blade elements are not taken into account. Please refer to the theory manual for details (blade bend-twist coupling).
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - FiniteElementBladeModelling
        - BladeModellingInsert
        - ModalBladeModelling
        - RigidBladeModelling
    
    """
    BladeModellingType: BladeModelling_BladeModellingTypeEnum = Field(alias="BladeModellingType", default=None)
    GeometricStiffnessModel: BladeModelling_GeometricStiffnessModelEnum = Field(alias="GeometricStiffnessModel", default=None)
    IgnoreAxesOrientationDifferencesForShear: bool = Field(alias="IgnoreAxesOrientationDifferencesForShear", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


BladeModelling.update_forward_refs()
